const { RuleTester } = require('eslint');
const rule = require('../../../lib/rules/no-new-ref-on-selector-functions'); // Replace with the actual path to your rule

const ruleTester = new RuleTester({
  parserOptions: { ecmaVersion: 2018 }
});

ruleTester.run('no-new-objects-or-arrays-in-selectors', rule, {
  valid: [
    // Using a pre-defined constant
    {
      code: `
        const dummyObj = {};
        const selector = (state) => state.jos || dummyObj;
      `
    },
    {
      code: `
        const dummyArr = [];
        function selector(state) {
          return state.jos || dummyArr;
        }
      `
    },
    // Returning an existing object or array from state
    {
      code: `
        const selector = (state) => state.jos;
      `
    },
    {
      code: `
        function selector(state) {
          return state.jos ? state.jos : anotherObj;
        }
      `
    },
    {
      code: `
        function selector(state = dummyObj) {
            return state;
        }
        `
    },
    {
      code: `
        const selector = (state = dummyArray) => state;
        `
    },
    {
      code: `
        function selector(state, param = { key: 'value' }) {
  return state;
}
        `
    },
    {
      code: `
        const selector = (state, param = [{}]) => state;

        `
    },
    {
      code: `
      const selector = (state, param = [{}]) => state;

      `,
      options: [{ referenceCreatingMethods: ['slice'] }]
    }
  ],

  invalid: [
    // Directly returning a new object
    {
      code: `
        function selector(state) {
          return {};
        }
      `,
      errors: [{ messageId: 'noNewObjectArray' }]
    },
    {
      code: `
        const selector = (state) => ({});
      `,
      errors: [{ messageId: 'noNewObjectArray' }]
    },
    // Directly returning a new array
    {
      code: `
        function selector(state) {
          return [];
        }
      `,
      errors: [{ messageId: 'noNewObjectArray' }]
    },
    {
      code: `
        const selector = (state) => [];
      `,
      errors: [{ messageId: 'noNewObjectArray' }]
    },
    // Using a variable that references a new object
    {
      code: `
        function selector(state) {
          const obj = {};
          return obj;
        }
      `,
      errors: [{ messageId: 'noVariableObjectArray' }]
    },
    {
      code: `
        const selector = (state) => {
          const obj = {};
          return obj;
        };
      `,
      errors: [{ messageId: 'noVariableObjectArray' }]
    },
    // Logical OR with a new object
    {
      code: `
        function selector(state) {
          return state.jos || {};
        }
      `,
      errors: [{ messageId: 'noNewObjectArray' }]
    },
    {
      code: `
        function somethingElse(state) {
          return {};
        }
      `,
      errors: [{ messageId: 'noNewObjectArray' }]
    },
    {
      code: `
        const selector = (state) => state.jos || {};
      `,
      errors: [{ messageId: 'noNewObjectArray' }]
    },
    // Conditional operator with a new object
    {
      code: `
        function selector(state) {
          return state.jos ? state.jos : {};
        }
      `,
      errors: [{ messageId: 'noNewObjectArray' }]
    },
    {
      code: `
        const selector = (state) => state.jos ? state.jos : {};
      `,
      errors: [{ messageId: 'noNewObjectArray' }]
    },
    {
      code: `
        function selector(state = {}) {
  return state;
}

        `,
      errors: [{ messageId: 'noDefaultParamReference' }]
    },
    {
      code: `const selector = (state = []) => state;
        `,
      errors: [{ messageId: 'noDefaultParamReference' }]
    },
    {
      code: `const selector = (state =[]) => {
return state;

}`,
      errors: [{ messageId: 'noDefaultParamReference' }]
    },
    {
      code: `const selector = (state =[]) => {
return state;

}`,
      errors: [{ messageId: 'noDefaultParamReference' }],
      options: [{ referenceCreatingMethods: ['slice'] }]
    }
  ]
});
