/* eslint-disable */
/**
 * @fileoverview To show error when followed bad practices inside mapStateToProps of react redux applications
 * @author Sivanesh
 */
'use strict';

//------------------------------------------------------------------------------
// Requirements
//------------------------------------------------------------------------------

var rule = require('../../../lib/rules/no-ref-mapstatetoprops'),
  RuleTester = require('eslint').RuleTester;

//------------------------------------------------------------------------------
// Tests
//------------------------------------------------------------------------------

var ruleTester = new RuleTester({ parserOptions: { ecmaVersion: 2018 } });
const ERROR_MESSAGE =
  'Causes performance issue as it creates a new reference each time when redux calls this connect.';

const ARRAY_REF_ERR_MESSAGE =
  'Creates new Array reference. Hence causes performance issues.';

const OBJ_REF_ERR_MSG = `Creates new Object reference. Hence causes performance issues.`;
ruleTester.run('no-ref-mapstatetoprops', rule, {
  valid: [
    'function mapStateToProps(state) {return {id: state.agent.id}}',
    'const mapStateToProps = state => {return {arr: state.arr}}',
    'connect(function(state) {return {id: state.module.id, name: state.name}})',
    'connect(state => {return {arr: state.arr}})',
    'connect(state => ({}))',
    'function mapStateToProps(state) { let { preference = {} } = state.userObj; return {mode: state.mode}}',
    'const mapStateToProps = (state) => { let { preference = {} } = state.userObj; return {mode: state.mode}}',
    'connect(state => {let a = arr.slice(); return {arr: state.arr}})',
    'connect(state => {let a = Object.assign(); return {arr: state.arr}})',
    'function mapStateToProps(state) { let { preference = {} } = state.userObj || []; return {mode: state.mode}}',
    `connect(function mapStateToProps (state) {
      let a = arr.slice();
    let b = Object.assign();
    	return {}
    })`,
    'connect(state => ({}))',
    `connect(state => {
      let a = arr.slice();
    let b = Object.assign();
    	return {}
    })`,
    `function mapStateToProps (state) {
      let a = arr.slice();
    let b = Object.assign();
      return {
      }
    }`,
    `const mapStateToProps = state => {
      let a = arr.slice();
    	let b = Object.assign();
      return {}
    }
    `,
    `const mapStateToProps = state =>  ({name: 1})`,
    {
      code: `const mapStateToProps = state =>  ({name: 1})`,
      options: [{arrRefs: ['slice'], objRefs: ['assign']}],
    }

    //
  ],
  invalid: [
    {
      code: 'function mapStateToProps(state) {return {mode: []}}',
      errors: [
        {
          message: ERROR_MESSAGE,
          type: 'ArrayExpression'
        }
      ]
    },
    {
      code: 'connect((state) => { return { mode: state.mode || {} } })',
      errors: [
        {
          message: ERROR_MESSAGE,
          type: 'ObjectExpression'
        }
      ]
    },
    {
      code: 'function mapStateToProps(state) {return {mode: state.mode || {}}}',
      errors: [
        {
          message: ERROR_MESSAGE,
          type: 'ObjectExpression'
        }
      ]
    },
    {
      // Array and Object references.
      code: `function mapStateToProps(state) {
        return {
          filtered: state.name.slice(1),
          newObj: Object.assign({}, state.module)
        }
      }
      `,
      errors: [
        {
          message: ARRAY_REF_ERR_MESSAGE
        },
        {
          message: OBJ_REF_ERR_MSG
        }
      ]
    },
    {
      // Array and Object references.
      code: `const mapStateToProps = (state) => {
        return {
          filtered: state.name.slice(1),
          newObj: Object.assign({}, state.module)
        }
      }
      `,
      errors: [
        {
          message: ARRAY_REF_ERR_MESSAGE
        },
        {
          message: OBJ_REF_ERR_MSG
        }
      ]
    },
    {
      // Array and Object references.
      code: `const mapStateToProps = (state) => ({
          filtered: state.name.slice(1),
          newObj: Object.assign({}, state.module)
        })

      `,
      errors: [
        {
          message: ARRAY_REF_ERR_MESSAGE
        },
        {
          message: OBJ_REF_ERR_MSG
        }
      ]
    },
    {
      // Array and Object references.
      code: `connect((state) => ({
          filtered: state.name.slice(1),
          newObj: Object.assign({}, state.module)
        }))

      `,
      errors: [
        {
          message: ARRAY_REF_ERR_MESSAGE
        },
        {
          message: OBJ_REF_ERR_MSG
        }
      ]
    },
    {
      code: `connect((state) => ({
          filtered: state.name.slice(1),
          newObj: Object.assign({}, state.module)
        }))`,
        options: [{arrRefs: ['slice'], objRefs: ['assign']}],
        errors: [
          {
            message: ARRAY_REF_ERR_MESSAGE
          },
          {
            message: OBJ_REF_ERR_MSG
          }
        ]
    },
  ]
});
