const { RuleTester } = require('eslint');
const rule = require('../../../lib/rules/no-store');

const ruleTester = new RuleTester({
  parserOptions: { ecmaVersion: 2018 }
});

const errorMessage = 'mapStateToProps should not return complete store object';

ruleTester.run('no-store', rule, {
  valid: [
    {
      code: `useSelector(state => state.value)`
    },
    {
      code: `
            useSelector(state => getValueFromState(state))
            `
    },
    {
      code: `
                useSelector(state => {
                    
                    return state.v;
                    
                    })
            `
    },
    {
      code: `function mapStateToProps(state) {
                return {
                s: state.value}
            }`
    },
    {
      code: `function mapStateToProps(state) {
                return {
                s: getValue(state)}
            }`
    },
    {
      code: `const mapStateToProps = (state) => {
                return {
                s: state.value}
            }`
    },
    {
      code: `connect((state) => ({v: state.value}))`
    },
    {
       code: `connect((state) => ({v: state.value}))`,
      options:[{hook: 'useSelector'}]
    }
  ],
  invalid: [
    {
      code: `
            useSelector(state => state)`,
      errors: [{ message: errorMessage }]
    },
    {
      code: `useSelector(state => ({...state}))`,
      errors: [{ message: errorMessage }]
    },
    {
      code: `useSelector(state => ({state}))`,
      errors: [{ message: errorMessage }]
    },
    {
      code: `useSelector(state => {
            return state
            })`,
      errors: [{ message: errorMessage }]
    },
    {
        code: `
        function mapStateToProps(state) {
        return {...state}
        } `,
        errors: [{message: errorMessage}]
    },
    {
        code: `
        function mapStateToProps(state) {
        return state
        } `,
        errors: [{message: errorMessage}]
    },
    {
        code: `
        const mapStateToProps = (state) => {
        return {...state}
        } `,
        errors: [{message: errorMessage}]
    }, 
    {
        code: `
        const mapStateToProps = (state) => {
        return state
        } `,
        errors: [{message: errorMessage}]
    }, 
    {
        code: `
        const mapStateToProps = (state) => state `,
        errors: [{message: errorMessage}]
    },
    , 
    {
        code: `const mapStateToProps = (state) => ({state}) `,
        errors: [{message: errorMessage}]
    },
    {
        code: `connect((state) => state)`,
        errors: [{message: errorMessage}]
    },
    {
        errors: [{message: errorMessage}],
        code: `connect(state => ({state}))`
    },
    {
        errors: [{message: errorMessage}],
        code: `connect(state => ({...state}))`
    },
    {
        errors: [{message: errorMessage}],
        code: `connect(function mapStateToProps(state) {
        return state
        })`
    },
    {
      errors: [{message: errorMessage}],
      code: `connect(function mapStateToProps(state) {
      return state
      })`,
      options:[{hook: 'useSelector'}]
  }
  ]
});
